@@ -1,6 +1,3 @@
-from numpy.core.fromnumeric import diagonal
-from pandas.core.algorithms import unique
-from collections import namedtuple
 from .block import Block
 from .face import Face, create_face_from_diagonals, split_face
 import math 
@@ -10,7 +7,6 @@ import numpy as np
 from .differencing import find_face_edges
 import pandas as pd
 from operator import eq 
-import tqdm
 from typing import List, NamedTuple
 import math
 from .point_match import point_match
@@ -78,7 +74,7 @@ def get_outer_faces(block1:Block):
     for i in range(len(faces)):
         matchFound = False
         for j in range(len(faces)):
-            if (i!=j and faces[i] == faces[j]):
+            if (i!=j and faces[i].vertices_equals(faces[j])):
                 matching.append((i,j))
                 matchFound = True
         if not matchFound:
@@ -106,17 +102,14 @@ def unique_pairs(listOfItems:list):
             yield x,y
 
 
-def find_matching_blocks(block1:Block,block2:Block):  
+def find_matching_blocks(block1:Block,block2:Block,tol:float=5E-5):  
     """Takes two blocks and finds all matching pairs
-
     Args:
         block1 (Block): Any plot3d Block that is not the same as block2
         block2 (Block): Any plot3d Block that is not the same as block1
         full_face_match (bool): (Depreciated) use full face matching (Much faster) Full face match can be deceiving. There could be some cases where the face is a wrap and 4 corners match but the insides do not. This kind of connection shouldn't be considered
-
     Returns:
         (tuple): containing
-
             - **df** (pandas.DataFrame): corners of matching pair as block1_corners,block2_corners ([imin,jmin,kmin],[imax,jmax,kmax]), ([imin,jmin,kmin],[imax,jmax,kmax])
             - **block1_outer** (List[Face]):
             - **block2_outer** (List[Face]): 
@@ -126,6 +119,7 @@ def find_matching_blocks(block1:Block,block2:Block):
 
     block1_outer,_ = get_outer_faces(block1)
     block2_outer,_ = get_outer_faces(block2)
+
     block1_split_faces = list()
     block2_split_faces = list() 
     block1MatchingFace = list()
@@ -143,7 +137,7 @@ def find_matching_blocks(block1:Block,block2:Block):
             block1_face = block1_outer[p]
             for q in range(len(block2_outer)):
                 block2_face = block2_outer[q]
-                df,split_faces1,split_faces2 = get_face_intersection(block1_face,block2_face,block1,block2)
+                df, split_faces1, split_faces2 = get_face_intersection(block1_face,block2_face,block1,block2,tol)
                 if len(df)>0:   # the number of intersection points has to be more than 4
                     # if not block1_face in block1MatchingFace and not block2_face in block2MatchingFace:
                     block_match_indices.append(df)
@@ -152,7 +146,10 @@ def find_matching_blocks(block1:Block,block2:Block):
                     block1_split_faces.extend(split_faces1)
                     block2_split_faces.extend(split_faces2)
                     match = True
-                    
+                    break
+            if match:
+                break
+
         [block1_outer.remove(b) for b in block1MatchingFace if b in block1_outer]
         [block2_outer.remove(b) for b in block2MatchingFace if b in block2_outer]
         if len(block1_split_faces)>0:
@@ -163,11 +160,15 @@ def find_matching_blocks(block1:Block,block2:Block):
             block2_split_faces.clear()
 
     return block_match_indices, block1_outer, block2_outer # Remove duplicates using set and list 
+                    
+        
+        
 
 
 def select_multi_dimensional(T:np.ndarray,dim1:tuple,dim2:tuple, dim3:tuple):
     """Takes a block (T) and selects X,Y,Z from the block given a face's dimensions
         theres really no good way to do this in python 
+        
     Args:
         T (np.ndarray): arbitrary array so say a full matrix containing X
         dim1 (tuple): 20,50 this selects X in the i direction from i=20 to 50
@@ -176,6 +177,7 @@ def select_multi_dimensional(T:np.ndarray,dim1:tuple,dim2:tuple, dim3:tuple):
 
     Returns:
         np.ndarray: returns X or Y or Z given some range of I,J,K
+
     """
     if dim1[0] == dim1[1]:
         return T[ dim1[0], dim2[0]:dim2[1]+1, dim3[0]:dim3[1]+1 ]
@@ -188,6 +190,7 @@ def select_multi_dimensional(T:np.ndarray,dim1:tuple,dim2:tuple, dim3:tuple):
 
 def get_face_intersection(face1:Face,face2:Face,block1:Block,block2:Block,tol:float=1E-6):
     """Get the index of the intersection between two faces located on two different blocks 
+        Face1 needs to be the smaller face. 
 
     Args:
         face1 (Face): An exterior face
@@ -209,30 +212,13 @@ def get_face_intersection(face1:Face,face2:Face,block1:Block,block2:Block,tol:fl
     split_faces1 = list()
     split_faces2 = list()
     
-    # See if we can speed up the match by looking at the match indicies 
-    matchedIndicies = face1.match_indices(face2) # face1 == face2, Full face match is automatically checked first. If this is not 4 then code proceeds to partial match
-    if len(matchedIndicies)>0: # Checks to see if the two faces corners are actually 
-        for i,j in matchedIndicies:
-            i1,j1,k1 = face1.I[i],face1.J[i],face1.K[i]
-            i2,j2,k2 = face2.I[j],face2.J[j],face2.K[j]
-            match_location.append({"i1":i1,"j1":j1,"k1":k1,'i2':i2,'j2':j2,'k2':k2})
-        df_temp = pd.DataFrame(match_location)
-        I1 = [df_temp['i1'].min(), df_temp['i1'].max()]
-        J1 = [df_temp['j1'].min(), df_temp['j1'].max()]
-        K1 = [df_temp['k1'].min(), df_temp['k1'].max()]
+    I1 = [face1.IMIN,face1.IMAX]
+    J1 = [face1.JMIN,face1.JMAX]
+    K1 = [face1.KMIN,face1.KMAX]
 
-        I2 = [df_temp['i2'].min(), df_temp['i2'].max()]
-        J2 = [df_temp['j2'].min(), df_temp['j2'].max()]
-        K2 = [df_temp['k2'].min(), df_temp['k2'].max()]
-    else:
-        # Check all points interior of the block
-        I1 = [face1.IMIN,face1.IMAX]
-        J1 = [face1.JMIN,face1.JMAX]
-        K1 = [face1.KMIN,face1.KMAX]
-
-        I2 = [face2.IMIN,face2.IMAX]
-        J2 = [face2.JMIN,face2.JMAX]
-        K2 = [face2.KMIN,face2.KMAX]
+    I2 = [face2.IMIN,face2.IMAX]
+    J2 = [face2.JMIN,face2.JMAX]
+    K2 = [face2.KMIN,face2.KMAX]
     
     # Grab the points of Face 1 and Face 2
     X1 = select_multi_dimensional(block1.X, (I1[0],I1[1]),(J1[0],J1[1]),(K1[0],K1[1]))
@@ -472,10 +458,8 @@ def find_block_index_in_outer_faces(outer_faces:List[Face], block_indx:int):
 
 def connectivity(blocks:List[Block]):
     """Returns a dictionary outlining the connectivity of the blocks along with any exterior surfaces 
-
     Args:
         blocks (List[Block]): List of all blocks in multi-block plot3d mesh
-
     Returns:
         (List[Dict]): All matching faces formatted as a list of { 'block1': {'block_index', 'IMIN', 'JMIN','KMIN', 'IMAX','JMAX','KMAX'} }
         (List[Dict]): All exterior surfaces formatted as a list of { 'block_index', 'surfaces': [{'IMIN', 'JMIN','KMIN', 'IMAX','JMAX','KMAX', 'ID'}] }
@@ -506,16 +490,14 @@ def connectivity(blocks:List[Block]):
                 matches_to_remove.append(create_face_from_diagonals(block=blocks[j],imin=df['i2'].min(),jmin=df['j2'].min(),kmin=df['k2'].min(),
                                                                                     imax=df['i2'].max(),jmax=df['j2'].max(),kmax=df['k2'].max()))
                 matches_to_remove[-1].set_block_index(j)
+                
+                face1 = matches_to_remove[-2]
+                face2 = matches_to_remove[-1]
 
-                face_matches.append({'block1':{
-                                            'block_index':i,'IMIN':df['i1'].min(),'JMIN':df['j1'].min(),'KMIN':df['k1'].min(),
-                                            'IMAX':df['i1'].max(),'JMAX':df['j1'].max(),'KMAX':df['k1'].max()
-                                        },
-                                    'block2':{
-                                            'block_index':j,'IMIN':df['i2'].min(),'JMIN':df['j2'].min(),'KMIN':df['k2'].min(),
-                                            'IMAX':df['i2'].max(),'JMAX':df['j2'].max(),'KMAX':df['k2'].max()
-                                        },
-                                    'match':df})
+                temp = face_matches_to_dict(face1,face2,blocks[i],blocks[j])
+                temp['match'] = df
+                # ! Paht check 
+                face_matches.append(temp)
 
         # Update Outer Faces 
         outer_faces.extend(blocki_outerfaces)
@@ -571,4 +553,86 @@ def connectivity(blocks:List[Block]):
                             'id':id, 'block_index':face.BlockIndex })
         id += 1
 
-    return face_matches, outer_faces_formatted 
+    return face_matches, outer_faces_formatted  
+
+
+def face_matches_to_dict(face1:Face, face2:Face,block1:Block,block2:Block):
+    """Makes sure the diagonal of face 1 match the diagonal of face 2
+
+    Args:
+        face1 (Face): Face 1 with block index 
+        face2 (Face): Face 2 with block index
+        block1 (Block): Block 1 
+        block2 (Block): Block 2 
+    
+    Returns:
+        (dict): dictionary describing the corner matches 
+
+    """
+    match = {
+            'block1':{
+                            'block_index':face1.BlockIndex,
+                            'IMIN':-1,'JMIN':-1,'KMIN':-1,  # Lower Corner
+                            'IMAX':-1,'JMAX':-1,'KMAX':-1   # Upper Corner
+                        },
+                'block2':{
+                            'block_index':face2.BlockIndex,
+                            'IMIN':-1,'JMIN':-1,'KMIN':-1,  # Lower Corner
+                            'IMAX':-1,'JMAX':-1,'KMAX':-1   # Upper Corner
+                        }
+                }
+            
+    I1 = [face1.IMIN,face1.IMAX]
+    J1 = [face1.JMIN,face1.JMAX]
+    K1 = [face1.KMIN,face1.KMAX]
+
+    I2 = [face2.IMIN,face2.IMAX]
+    J2 = [face2.JMIN,face2.JMAX]
+    K2 = [face2.KMIN,face2.KMAX]
+
+    # Search for corners     
+    x1_l = block1.X[I1[0],J1[0],K1[0]]    # lower corner of block 1 
+    y1_l = block1.Y[I1[0],J1[0],K1[0]]
+    z1_l = block1.Z[I1[0],J1[0],K1[0]]
+    # Matches which corner in block 2 
+    search_results = list()
+    for p in I2: 
+        for q in J2:
+            for r in K2:
+                x2 = block2.X[p,q,r]  
+                y2 = block2.Y[p,q,r]
+                z2 = block2.Z[p,q,r]
+                dx = x2-x1_l; dy = y2-y1_l; dz = z2 -z1_l                
+                search_results.append({'I':p,'J':q,'K':r,'d':math.sqrt(dx*dx + dy*dy + dz*dz)})    
+    df = pd.DataFrame(search_results)
+    df = df.sort_values(by=['d'])
+    match['block1']['IMIN'] = face1.IMIN
+    match['block1']['JMIN'] = face1.JMIN
+    match['block1']['KMIN'] = face1.KMIN
+    match['block2']['IMIN'] = int(df.iloc[0]['I'])
+    match['block2']['JMIN'] = int(df.iloc[0]['J'])
+    match['block2']['KMIN'] = int(df.iloc[0]['K'])
+    
+    # Search for corners     
+    x1_u = block1.X[I1[1],J1[1],K1[1]]    # lower corner of block 1 
+    y1_u = block1.Y[I1[1],J1[1],K1[1]]
+    z1_u = block1.Z[I1[1],J1[1],K1[1]]
+    # Matches which corner in block 2 
+    search_results = list()
+    for p in I2: 
+        for q in J2:
+            for r in K2:
+                x2 = block2.X[p,q,r]  
+                y2 = block2.Y[p,q,r]
+                z2 = block2.Z[p,q,r]
+                dx = x2-x1_u; dy = y2-y1_u; dz = z2 -z1_u
+                search_results.append({'I':p,'J':q,'K':r,'d':math.sqrt(dx*dx + dy*dy + dz*dz)})
+    df = pd.DataFrame(search_results)
+    df = df.sort_values(by=['d'])
+    match['block1']['IMAX'] = face1.IMAX
+    match['block1']['JMAX'] = face1.JMAX
+    match['block1']['KMAX'] = face1.KMAX
+    match['block2']['IMAX'] = int(df.iloc[0]['I'])
+    match['block2']['JMAX'] = int(df.iloc[0]['J'])
+    match['block2']['KMAX'] = int(df.iloc[0]['K'])
+    return match
\ No newline at end of file
